import { __awaiter, __decorate } from "tslib";
import { Connect, Datasource, normaliseCriteria, dataServerResultFilter, MessageType, } from '@genesislcap/foundation-comms';
import { customElement, FASTElement, observable } from '@microsoft/fast-element';
import { AlertConfig } from '../config/config';
import { FoundationAlertsStyles as styles } from '../styles';
import { logger } from '../utils';
import { FoundationAlertsTemplate as template } from './main.template';
/**
 * Foundation alerts component for handle alerts rules list
 * Allow creating, disabling and deleting alerts
 * @beta
 */
export class FoundationAlerts extends FASTElement {
    constructor() {
        super(...arguments);
        this.displayState = 'list';
        this.items = [];
        this.resourceName = 'ALL_DYNAMIC_NOTIFY_RULES';
        this.statusLabelMap = {
            ENABLED: 'Active',
            DISABLED: 'Paused',
        };
        this.ready = false;
        this.filters = [
            {
                name: 'All',
                callback: () => {
                    this.removeFilter('RULE_STATUS');
                },
                icon: {
                    name: 'reply-all',
                },
            },
            {
                name: 'Active',
                callback: () => {
                    this.setFilter('RULE_STATUS', 'ENABLED');
                },
                icon: {
                    name: 'check',
                },
            },
            {
                name: 'Inactive',
                callback: () => {
                    this.setFilter('RULE_STATUS', 'DISABLED');
                },
                icon: {
                    name: 'xmark',
                },
            },
        ];
        this.criteriaFromFilters = new Map();
    }
    criteriaFromFiltersChanged() {
        if (this.ready) {
            this.fetchData();
        }
    }
    connectedCallback() {
        const _super = Object.create(null, {
            connectedCallback: { get: () => super.connectedCallback }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.connectedCallback.call(this);
            logger.debug(`foundation-alerts is now connected to the DOM`);
            yield this.fetchData();
            this.ready = true;
        });
    }
    disconnectedCallback() {
        var _a, _b;
        super.disconnectedCallback();
        /**
         * Attempt to clean up after ourselves.
         */
        (_a = this.dataSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        delete this.dataSubscription;
        (_b = this.datasource) === null || _b === void 0 ? void 0 : _b.destroy();
        delete this.datasource;
    }
    findIndexAlertByRef(rowRef) {
        return this.items.findIndex((item) => item.ROW_REF === rowRef);
    }
    fetchData() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (this.dataSubscription) {
                this.dataSubscription.unsubscribe();
                this.dataSubscription = undefined;
                this.items = [];
            }
            if (this.sourceRef) {
                this.connect.dataLogoff(this.sourceRef);
            }
            const datasourceInit = yield this.datasource.init({
                criteria: this.buildCriteria(),
                resourceName: this.resourceName,
            }, true);
            if (!datasourceInit) {
                logger.debug(`Alerts rules Datasource init failed for ${this.resourceName}`);
                return;
            }
            logger.debug(`requesting stream for ${this.resourceName}`);
            this.dataSubscription = (_a = this.datasource.stream) === null || _a === void 0 ? void 0 : _a.subscribe((result) => {
                const messageType = result.MESSAGE_TYPE;
                this.sourceRef = result.SOURCE_REF;
                if (messageType && messageType === MessageType.LOGOFF_ACK) {
                    return;
                }
                if (result.ROW) {
                    this.handleStreamUpdate(dataServerResultFilter(result));
                }
                else {
                    this.items = [];
                }
            });
        });
    }
    handleStreamUpdate(dataServerResult) {
        var _a, _b, _c;
        if (((_a = dataServerResult.inserts) === null || _a === void 0 ? void 0 : _a.length) > 0) {
            this.handleInserts(dataServerResult.inserts);
        }
        if (((_b = dataServerResult.deletes) === null || _b === void 0 ? void 0 : _b.length) > 0) {
            this.handleDeletes(dataServerResult.deletes);
        }
        if (((_c = dataServerResult.updates) === null || _c === void 0 ? void 0 : _c.length) > 0) {
            this.handleUpdates(dataServerResult.updates);
        }
    }
    handleInserts(insertedRows) {
        this.items = [
            ...this.items,
            ...insertedRows.map((row) => (Object.assign(Object.assign({}, row), { DATE: new Date().toLocaleDateString() }))),
        ];
    }
    handleDeletes(deletedRows) {
        deletedRows.forEach((deletedRow) => {
            const alertIndex = this.findIndexAlertByRef(deletedRow.ROW_REF);
            if (alertIndex > -1) {
                this.items.splice(alertIndex, 1);
            }
        });
    }
    handleUpdates(updatedRows) {
        updatedRows.forEach((updatedRow) => {
            const alertIndex = this.findIndexAlertByRef(updatedRow.ROW_REF);
            if (alertIndex > -1) {
                this.items[alertIndex] = Object.assign(Object.assign({}, this.items[alertIndex]), updatedRow);
                this.items = [...this.items];
            }
        });
    }
    buildCriteria() {
        const criteriaList = [];
        this.criteriaFromFilters.forEach((fieldValue, fieldName) => {
            criteriaList.push(`${fieldName}=='${fieldValue}'`);
        });
        return normaliseCriteria(criteriaList.join(' && '), ';');
    }
    setFilter(fieldName, newFilter) {
        this.criteriaFromFilters = new Map(this.criteriaFromFilters.set(fieldName, newFilter));
    }
    removeFilter(fieldName) {
        this.criteriaFromFilters.delete(fieldName);
        this.criteriaFromFilters = new Map(this.criteriaFromFilters);
    }
    delete(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.connect.commitEvent('EVENT_DYNAMIC_RULE_NOTIFY_DELETE', {
                DETAILS: {
                    DYNAMIC_NOTIFY_RULE_ID: id,
                },
            });
        });
    }
    toogleStatus({ RULE_STATUS, DYNAMIC_NOTIFY_RULE_ID: ID, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const eventName = RULE_STATUS === 'ENABLED' ? 'EVENT_DYNAMIC_RULE_DISABLE' : 'EVENT_DYNAMIC_RULE_ENABLE';
            return this.connect.commitEvent(eventName, { DETAILS: { ID } });
        });
    }
    handleAddAlertAction() {
        this.displayState = 'add';
    }
    handleEditAlertAction(item) {
        this.alertEditItemData = Object.assign(Object.assign({}, item), { TABLE_ID: item.RULE_TABLE, USERNAME: item.USER_NAME });
        this.displayState = 'edit';
    }
}
__decorate([
    Connect
], FoundationAlerts.prototype, "connect", void 0);
__decorate([
    Datasource
], FoundationAlerts.prototype, "datasource", void 0);
__decorate([
    observable
], FoundationAlerts.prototype, "displayState", void 0);
__decorate([
    observable
], FoundationAlerts.prototype, "items", void 0);
__decorate([
    observable
], FoundationAlerts.prototype, "alertEditItemData", void 0);
__decorate([
    AlertConfig
], FoundationAlerts.prototype, "alertConfig", void 0);
__decorate([
    observable
], FoundationAlerts.prototype, "criteriaFromFilters", void 0);
/**
 * ZeroAlerts.
 *
 * @remarks
 * A zero version that pre-registers zero components and uses the zero design system. ZeroAlerts and its components won't bloat the core bundle as
 * these are only loaded on-demand when in use.
 *
 * @example
 * ```ts
 * import { ZeroAlerts } from '@genesislcap/foundation-alerts';
 * ...
 * ZeroAlerts
 * ```
 *
 * @example Load the micro frontend on-demand
 * ```ts

 *   const { ZeroAlerts } = await import('@genesislcap/foundation-alerts');
 *   return ZeroAlerts;
 * },
 * ```
 *
 * @public
 */
let ZeroAlerts = class ZeroAlerts extends FoundationAlerts {
    /**
     * @internal
     */
    connectedCallback() {
        const _super = Object.create(null, {
            connectedCallback: { get: () => super.connectedCallback }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.connectedCallback.call(this);
            yield this.loadRemotes();
        });
    }
    /**
     * @internal
     */
    loadRemotes() {
        return __awaiter(this, void 0, void 0, function* () {
            const { registerCommonZeroComponents } = yield import('../components/zero-components');
            yield registerCommonZeroComponents();
        });
    }
};
ZeroAlerts = __decorate([
    customElement({
        name: 'zero-alerts',
        template,
        styles,
    })
], ZeroAlerts);
export { ZeroAlerts };
